@@ -10,8 +10,8 @@ T = TypeVar('T')
 RegistryValue = TypeVar('RegistryValue')
 
 from neutronapi.base import API, Response
-from neutronapi import exceptions
-from neutronapi.middleware.cors import CORS
+from neutronapi.api import exceptions
+from neutronapi.middleware.cors import CorsMiddleware
 from neutronapi.middleware.routing import RoutingMiddleware
 from neutronapi.middleware.allowed_hosts import AllowedHostsMiddleware
 
@@ -104,19 +104,40 @@ class Application:
             >>> logger = self.registry.get('utils:logger')
             >>> email = self.registry.get('services:email')
         """
-        # Convert provided APIs (list or dict) into internal {resource: api} mapping
+        # Convert provided APIs into two mappings: name-based and resource-based
         from neutronapi.base import API
-        self.apis: Dict[str, 'API'] = {}
+        self.apis: Dict[str, 'API'] = {}  # name -> api (for reverse lookups)
+        self._resource_apis: Dict[str, 'API'] = {}  # resource -> api (for routing)
+        
         if apis:
-            # Support both list[API] and dict[str, API]
-            api_iterable = apis.values() if isinstance(apis, dict) else apis
-            for api in api_iterable:
-                if not hasattr(api, 'resource'):
-                    raise ValueError(f"API {api.__class__.__name__} must have a 'resource' attribute")
-                resource = getattr(api, 'resource', None)
-                if resource is None:
-                    raise ValueError(f"API {api.__class__.__name__} must define a non-null 'resource'")
-                self.apis[resource] = api
+            if isinstance(apis, dict):
+                # Use the provided names from the dict
+                for name, api in apis.items():
+                    if not hasattr(api, 'resource'):
+                        raise ValueError(f"API {api.__class__.__name__} must have a 'resource' attribute")
+                    resource = getattr(api, 'resource', None)
+                    if resource is None:
+                        raise ValueError(f"API {api.__class__.__name__} must define a non-null 'resource'")
+                    self.apis[name] = api
+                    self._resource_apis[resource] = api
+            else:
+                # For list[API], use the API name as the key for reverse lookups
+                for api in apis:
+                    if not hasattr(api, 'resource'):
+                        raise ValueError(f"API {api.__class__.__name__} must have a 'resource' attribute")
+                    resource = getattr(api, 'resource', None)
+                    if resource is None:
+                        raise ValueError(f"API {api.__class__.__name__} must define a non-null 'resource'")
+                    
+                    # Require explicit 'name' attribute for reverse lookups
+                    if not hasattr(api, 'name') or not api.name:
+                        raise ValueError(f"API {api.__class__.__name__} must have a 'name' attribute for reverse lookups")
+                    
+                    self.apis[api.name] = api
+                    self._resource_apis[resource] = api
+        
+        # Validate no duplicate route names across all APIs
+        self._validate_unique_route_names()
 
         self.version = version
 
@@ -141,13 +162,13 @@ class Application:
                 path = scope.get("path", "/")
 
                 # Check if path matches any API exactly
-                if path in self.apis:
-                    api = self.apis[path]
+                if path in self._resource_apis:
+                    api = self._resource_apis[path]
                     await api.handle(scope, receive, send)
                     return
 
                 # Check if path starts with any API prefix
-                for api_path, api in self.apis.items():
+                for api_path, api in self._resource_apis.items():
                     if path.startswith(api_path):
                         await api.handle(scope, receive, send)
                         return
@@ -188,7 +209,7 @@ class Application:
             # Legacy minimal wrapping
             hosts_app = AllowedHostsMiddleware(base_router,
                                                allowed_hosts=allowed_hosts) if allowed_hosts else base_router
-            self.app = CORS(hosts_app, allow_all_origins=cors_allow_all)
+            self.app = CorsMiddleware(hosts_app, allow_all_origins=cors_allow_all)
 
         # lifecycle hooks
         self.app.on_startup = []
@@ -329,6 +350,65 @@ class Application:
         prefix = f"{namespace}:"
         return [key for key in self.registry.keys() if key.startswith(prefix)]
 
+    def _validate_unique_route_names(self) -> None:
+        """Validate that there are no duplicate route names across all APIs.
+        
+        Route names are in the format "api_name:endpoint_name".
+        
+        Raises:
+            ValueError: If duplicate route names are found
+        """
+        seen_routes = {}  # route_name -> api_name
+        
+        for api_name, api in self.apis.items():
+            # Get all route names from this API
+            for route_tuple in api.routes:
+                # Route tuple structure: (pattern, handler, methods, permissions, throttles, route_name, original_path, ...)
+                if len(route_tuple) >= 6:
+                    route_name = route_tuple[5]  # route_name is at index 5
+                    if route_name:  # Only check named routes
+                        full_route_name = f"{api_name}:{route_name}"
+                        
+                        if full_route_name in seen_routes:
+                            raise ValueError(
+                                f"Duplicate route name '{full_route_name}' found. "
+                                f"Route names must be unique across all APIs. "
+                                f"Previously defined in API '{seen_routes[full_route_name]}', "
+                                f"now found again in API '{api_name}'."
+                            )
+                        
+                        seen_routes[full_route_name] = api_name
+
+    def reverse(self, name: str, **kwargs) -> str:
+        """Reverse URL lookup for a named route across all registered APIs.
+        
+        Args:
+            name: Route name in format "api_name:endpoint_name"
+            **kwargs: Parameters to substitute in the URL
+            
+        Returns:
+            The reversed URL string
+            
+        Raises:
+            ValueError: If API not found or invalid name format
+            
+        Example:
+            >>> app = Application(apis={"users": users_api})
+            >>> url = app.reverse("users:detail", user_id=123)
+            >>> # Returns: "/users/123"
+        """
+        if ":" not in name:
+            raise ValueError(
+                f"Route name '{name}' must be in format 'api_name:endpoint_name'. "
+                f"Example: 'users:detail'"
+            )
+        
+        api_name, endpoint_name = name.split(":", 1)
+        if api_name not in self.apis:
+            raise ValueError(f"API '{api_name}' not found.")
+        
+        return self.apis[api_name].reverse(endpoint_name, **kwargs)
+
     async def __call__(self, scope, receive, send, **kwargs):
         return await self.app(scope, receive, send, **kwargs)
 
